@@ -24,30 +24,33 @@ Kaldi engine classes
 
 import collections, logging, os, sys, time
 
-from packaging.version import Version
-from six import string_types, print_, reraise
-from six.moves import zip
 import kaldi_active_grammar
-from kaldi_active_grammar       import KaldiError, KaldiRule
+from packaging.version     import Version
+from six                   import string_types, print_, reraise
+from six.moves             import zip
+from kaldi_active_grammar  import KaldiError, KaldiRule
 
-from ..base                     import (EngineBase,
-                                        EngineError,
-                                        MimicFailure,
-                                        DelegateTimerManager,
-                                        DelegateTimerManagerInterface,
-                                        DictationContainerBase,
-                                        GrammarWrapperBase)
-from .audio                     import MicAudio, VADAudio, AudioStore, WavAudio
-from .dictation                 import user_dictation_list, user_dictation_dictlist
-from .recobs                    import KaldiRecObsManager
-from .testing                   import debug_timer
-from dragonfly.grammar.state    import State
-from dragonfly.windows          import Window
+from dragonfly.grammar.state   import State
+from dragonfly.windows.window  import Window
+from dragonfly.engines.base    import (EngineBase,
+                                       EngineError,
+                                       MimicFailure,
+                                       DelegateTimerManager,
+                                       DelegateTimerManagerInterface,
+                                       DictationContainerBase,
+                                       GrammarWrapperBase)
+from dragonfly.engines.backend_kaldi.audio      import (MicAudio, VADAudio,
+                                                        AudioStore,
+                                                        WavAudio)
+from dragonfly.engines.backend_kaldi.dictation  import (user_dictation_list,
+                                                        user_dictation_dictlist)
+from dragonfly.engines.backend_kaldi.recobs     import KaldiRecObsManager
+from dragonfly.engines.backend_kaldi.testing    import debug_timer
 
 # Import the Kaldi compiler class. Suppress metaclass TypeErrors raised
 # during documentation builds caused by mocking KAG.
 try:
-    from .compiler                  import KaldiCompiler
+    from dragonfly.engines.backend_kaldi.compiler import KaldiCompiler
 except TypeError:
     if os.environ.get("SPHINX_BUILD_RUNNING"):
         KaldiCompiler = None
@@ -63,7 +66,8 @@ class KaldiEngine(EngineBase, DelegateTimerManagerInterface):
 
     _name = "kaldi"
     DictationContainer = DictationContainerBase
-    _required_kag_version = "2.1.0"
+    # NOTE: Remember to also update setup.py to the same version!
+    _required_kag_version = "3.1.0"
 
     #-----------------------------------------------------------------------
 
@@ -71,7 +75,8 @@ class KaldiEngine(EngineBase, DelegateTimerManagerInterface):
         audio_input_device=None, audio_self_threaded=True, audio_auto_reconnect=True, audio_reconnect_callback=None,
         retain_dir=None, retain_audio=None, retain_metadata=None, retain_approval_func=None,
         vad_aggressiveness=3, vad_padding_start_ms=150, vad_padding_end_ms=200, vad_complex_padding_end_ms=600,
-        auto_add_to_user_lexicon=False, lazy_compilation=True, invalidate_cache=False,
+        auto_add_to_user_lexicon=True, allow_online_pronunciations=False,
+        lazy_compilation=True, invalidate_cache=False,
         expected_error_rate_threshold=None,
         alternative_dictation=None,
         compiler_init_config=None, decoder_init_config=None,
@@ -129,6 +134,7 @@ class KaldiEngine(EngineBase, DelegateTimerManagerInterface):
             vad_padding_end_ms = int(vad_padding_end_ms),
             vad_complex_padding_end_ms = int(vad_complex_padding_end_ms),
             auto_add_to_user_lexicon = bool(auto_add_to_user_lexicon),
+            allow_online_pronunciations = bool(allow_online_pronunciations),
             lazy_compilation = bool(lazy_compilation),
             invalidate_cache = bool(invalidate_cache),
             expected_error_rate_threshold = float(expected_error_rate_threshold) if expected_error_rate_threshold is not None else None,
@@ -169,6 +175,7 @@ class KaldiEngine(EngineBase, DelegateTimerManagerInterface):
 
         self._compiler = KaldiCompiler(self._options['model_dir'], tmp_dir=self._options['tmp_dir'],
             auto_add_to_user_lexicon=self._options['auto_add_to_user_lexicon'],
+            allow_online_pronunciations=self._options['allow_online_pronunciations'],
             lazy_compilation=self._options['lazy_compilation'],
             alternative_dictation=self._options['alternative_dictation'],
             **self._options['compiler_init_config']
@@ -425,7 +432,7 @@ class KaldiEngine(EngineBase, DelegateTimerManagerInterface):
                             if kaldi_rule and is_acceptable_recognition:  # Don't store audio/metadata for bad recognitions
                                 self.audio_store.finalize(parsed_output,
                                     kaldi_rule.parent_grammar.name, kaldi_rule.parent_rule.name,
-                                    likelihood=expected_error_rate, has_dictation=kaldi_rule.has_dictation)
+                                    likelihood=expected_error_rate, has_dictation=recognition.has_dictation)
                             else:
                                 self.audio_store.cancel()
 
